"use strict";
exports.__esModule = true;
var thinEngine_1 = require("../../Engines/thinEngine");
var internalTexture_1 = require("../../Materials/Textures/internalTexture");
thinEngine_1.ThinEngine.prototype.createDynamicTexture = function (width, height, generateMipMaps, samplingMode) {
    var texture = new internalTexture_1.InternalTexture(this, internalTexture_1.InternalTextureSource.Dynamic);
    texture.baseWidth = width;
    texture.baseHeight = height;
    if (generateMipMaps) {
        width = this.needPOTTextures ? thinEngine_1.ThinEngine.GetExponentOfTwo(width, this._caps.maxTextureSize) : width;
        height = this.needPOTTextures ? thinEngine_1.ThinEngine.GetExponentOfTwo(height, this._caps.maxTextureSize) : height;
    }
    //  this.resetTextureCache();
    texture.width = width;
    texture.height = height;
    texture.isReady = false;
    texture.generateMipMaps = generateMipMaps;
    texture.samplingMode = samplingMode;
    this.updateTextureSamplingMode(samplingMode, texture);
    this._internalTexturesCache.push(texture);
    return texture;
};
thinEngine_1.ThinEngine.prototype.updateDynamicTexture = function (texture, canvas, invertY, premulAlpha, format, forceBindTexture) {
    if (premulAlpha === void 0) { premulAlpha = false; }
    if (forceBindTexture === void 0) { forceBindTexture = false; }
    if (!texture) {
        return;
    }
    this._bindTextureDirectly(this._gl.TEXTURE_2D, texture, true, forceBindTexture);
    this._unpackFlipY(invertY);
    if (premulAlpha) {
        this._gl.pixelStorei(this._gl.UNPACK_PREMULTIPLY_ALPHA_WEBGL, 1);
    }
    var internalFormat = format ? this._getInternalFormat(format) : this._gl.RGBA;
    this._gl.texImage2D(this._gl.TEXTURE_2D, 0, internalFormat, internalFormat, this._gl.UNSIGNED_BYTE, canvas);
    if (texture.generateMipMaps) {
        this._gl.generateMipmap(this._gl.TEXTURE_2D);
    }
    this._bindTextureDirectly(this._gl.TEXTURE_2D, null);
    if (premulAlpha) {
        this._gl.pixelStorei(this._gl.UNPACK_PREMULTIPLY_ALPHA_WEBGL, 0);
    }
    texture.isReady = true;
};
