"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
var renderTargetTexture_1 = require("../Textures/renderTargetTexture");
var internalTexture_1 = require("../Textures/internalTexture");
/**
 * Renders to multiple views with a single draw call
 * @see https://www.khronos.org/registry/webgl/extensions/WEBGL_multiview/
 */
var MultiviewRenderTarget = /** @class */ (function (_super) {
    __extends(MultiviewRenderTarget, _super);
    /**
     * Creates a multiview render target
     * @param scene scene used with the render target
     * @param size the size of the render target (used for each view)
     */
    function MultiviewRenderTarget(scene, size) {
        if (size === void 0) { size = 512; }
        var _this = _super.call(this, "multiview rtt", size, scene, false, true, internalTexture_1.InternalTextureSource.Unknown, false, undefined, false, false, true, undefined, true) || this;
        var internalTexture = scene.getEngine().createMultiviewRenderTargetTexture(_this.getRenderWidth(), _this.getRenderHeight());
        internalTexture.isMultiview = true;
        _this._texture = internalTexture;
        _this.samples = _this._engine.getCaps().maxSamples || _this.samples;
        return _this;
    }
    /**
     * @hidden
     * @param faceIndex the face index, if its a cube texture
     */
    MultiviewRenderTarget.prototype._bindFrameBuffer = function (faceIndex) {
        if (faceIndex === void 0) { faceIndex = 0; }
        if (!this._texture) {
            return;
        }
        this.getScene().getEngine().bindMultiviewFramebuffer(this._texture);
    };
    /**
     * Gets the number of views the corresponding to the texture (eg. a MultiviewRenderTarget will have > 1)
     * @returns the view count
     */
    MultiviewRenderTarget.prototype.getViewCount = function () {
        return 2;
    };
    return MultiviewRenderTarget;
}(renderTargetTexture_1.RenderTargetTexture));
exports.MultiviewRenderTarget = MultiviewRenderTarget;
