﻿#if defined(BUMP) || !defined(NORMAL) || defined(FORCENORMALFORWARD) || defined(SPECULARAA) || defined(CLEARCOAT_BUMP) || defined(ANISOTROPIC)
#extension GL_OES_standard_derivatives : enable
#endif

#ifdef LODBASEDMICROSFURACE
#extension GL_EXT_shader_texture_lod : enable
#endif

#define CUSTOM_FRAGMENT_BEGIN

#ifdef LOGARITHMICDEPTH
#extension GL_EXT_frag_depth : enable
#endif

precision highp float;

// Forces linear space for image processing
#ifndef FROMLINEARSPACE
    #define FROMLINEARSPACE
#endif

// Declaration
#include<__decl__pbrFragment>
#include<pbrFragmentExtraDeclaration>
#include<__decl__lightFragment>[0..maxSimultaneousLights]
#include<pbrFragmentSamplersDeclaration>
#include<imageProcessingDeclaration>
#include<clipPlaneFragmentDeclaration>
#include<logDepthDeclaration>
#include<fogFragmentDeclaration>

// Helper Functions
#include<helperFunctions>
#include<pbrHelperFunctions>
#include<imageProcessingFunctions>
#include<shadowsFragmentFunctions>
#include<harmonicsFunctions>
#include<pbrDirectLightingSetupFunctions>
#include<pbrDirectLightingFalloffFunctions>
#include<pbrBRDFFunctions>
#include<pbrDirectLightingFunctions>
#include<pbrIBLFunctions>
#include<bumpFragmentFunctions>

#ifdef REFLECTION
    #include<reflectionFunction>
#endif

#include<pbrBlockAlbedoOpacity>
#include<pbrBlockReflectivity>
#include<pbrBlockAmbientOcclusion>
#include<pbrBlockAlphaFresnel>
#include<pbrBlockAnisotropic>
#include<pbrBlockReflection>
#include<pbrBlockSheen>
#include<pbrBlockSubSurface>

#define CUSTOM_FRAGMENT_DEFINITIONS

// _____________________________ MAIN FUNCTION ____________________________
void main(void) {

    #define CUSTOM_FRAGMENT_MAIN_BEGIN

    #include<clipPlaneFragment>

    // _____________________________ Geometry Information ____________________________
    #include<pbrBlockNormalGeometric>

    #include<bumpFragment>

    #include<pbrBlockNormalFinal>

    // _____________________________ Albedo & Opacity ______________________________
    albedoOpacityOutParams albedoOpacityOut;

#ifdef ALBEDO
    vec4 albedoTexture = texture2D(albedoSampler, vAlbedoUV + uvOffset);
#endif

#ifdef OPACITY
    vec4 opacityMap = texture2D(opacitySampler, vOpacityUV + uvOffset);
#endif

    albedoOpacityBlock(
        vAlbedoColor,
    #ifdef ALBEDO
        albedoTexture,
        vAlbedoInfos,
    #endif
    #ifdef OPACITY
        opacityMap,
        vOpacityInfos,
    #endif
        albedoOpacityOut
    );

    vec3 surfaceAlbedo = albedoOpacityOut.surfaceAlbedo;
    float alpha = albedoOpacityOut.alpha;

    #define CUSTOM_FRAGMENT_UPDATE_ALBEDO

    #define CUSTOM_FRAGMENT_UPDATE_ALPHA

    #include<depthPrePass>

    #define CUSTOM_FRAGMENT_BEFORE_LIGHTS

    // _____________________________ AO  _______________________________
    ambientOcclusionOutParams aoOut;

#ifdef AMBIENT
    vec3 ambientOcclusionColorMap = texture2D(ambientSampler, vAmbientUV + uvOffset).rgb;
#endif

    ambientOcclusionBlock(
    #ifdef AMBIENT
        ambientOcclusionColorMap,
        vAmbientInfos,
    #endif
        aoOut
    );

#ifdef UNLIT
    vec3 diffuseBase = vec3(1., 1., 1.);
#else

    // _____________________________ Reflectivity _______________________________
    vec3 baseColor = surfaceAlbedo;

    reflectivityOutParams reflectivityOut;

    reflectivityBlock(
        vReflectivityColor,
        uvOffset,
    #ifdef METALLICWORKFLOW
        surfaceAlbedo,
    #endif
    #ifdef REFLECTIVITY
        vReflectivityInfos,
        vReflectivityUV,
        reflectivitySampler,
    #endif
    #if defined(METALLICWORKFLOW) && defined(REFLECTIVITY)  && defined(AOSTOREINMETALMAPRED)
        aoOut.ambientOcclusionColor,
    #endif
    #ifdef MICROSURFACEMAP
        vMicroSurfaceSamplerUV,
        vMicroSurfaceSamplerInfos,
        microSurfaceSampler,
    #endif
        reflectivityOut
    );

    float microSurface = reflectivityOut.microSurface;
    float roughness = reflectivityOut.roughness;

    #ifdef METALLICWORKFLOW
        surfaceAlbedo = reflectivityOut.surfaceAlbedo;
    #endif
    #if defined(METALLICWORKFLOW) && defined(REFLECTIVITY) && defined(AOSTOREINMETALMAPRED)
        aoOut.ambientOcclusionColor = reflectivityOut.ambientOcclusionColor;
    #endif

    // _____________________________ Alpha Fresnel ___________________________________
    #ifdef ALPHAFRESNEL
        #if defined(ALPHATEST) || defined(ALPHABLEND)
            alphaFresnelOutParams alphaFresnelOut;

            alphaFresnelBlock(
                normalW,
                viewDirectionW,
                alpha,
                microSurface,
                alphaFresnelOut
            );

            alpha = alphaFresnelOut.alpha;
        #endif
    #endif

    // _____________________________ Compute Geometry info _________________________________
    #include<pbrBlockGeometryInfo>

    // _____________________________ Anisotropy _______________________________________
    #ifdef ANISOTROPIC
        anisotropicOutParams anisotropicOut;

        anisotropicBlock(
            vAnisotropy,
            vAnisotropyInfos,
        #ifdef ANISOTROPIC_TEXTURE
            vAnisotropyUV,
            uvOffset,
            anisotropySampler,
        #endif
            TBN,
            normalW,
            viewDirectionW,
            anisotropicOut
        );
    #endif

    // _____________________________ Reflection Info _______________________________________
    #ifdef REFLECTION
        reflectionOutParams reflectionOut;

        reflectionBlock(
            vPositionW,
            normalW,
            alphaG,
            vReflectionMicrosurfaceInfos,
            vReflectionInfos,
        #ifdef ANISOTROPIC
            anisotropicOut,
        #endif
        #if defined(LODINREFLECTIONALPHA) && !defined(REFLECTIONMAP_SKYBOX)
            NdotVUnclamped,
        #endif
        #ifdef LINEARSPECULARREFLECTION
            roughness,
        #endif
            reflectionSampler,
        #if defined(NORMAL) && defined(USESPHERICALINVERTEX)
            vEnvironmentIrradiance,
        #endif
        #ifdef USESPHERICALFROMREFLECTIONMAP
            #if !defined(NORMAL) || !defined(USESPHERICALINVERTEX)
                reflectionMatrix,
            #endif
        #endif
        #ifdef USEIRRADIANCEMAP
            irradianceSampler,
        #endif
            reflectionOut
        );
    #endif

    // ___________________ Compute Reflectance aka R0 F0 info _________________________
    float reflectance = max(max(reflectivityOut.surfaceReflectivityColor.r, reflectivityOut.surfaceReflectivityColor.g), reflectivityOut.surfaceReflectivityColor.b);
    float reflectance90 = fresnelGrazingReflectance(reflectance);
    vec3 specularEnvironmentR0 = reflectivityOut.surfaceReflectivityColor.rgb;
    vec3 specularEnvironmentR90 = vec3(1.0, 1.0, 1.0) * reflectance90;

    // ________________________________ Sheen ______________________________
    #ifdef SHEEN
        sheenOutParams sheenOut;

        sheenBlock(
            vSheenColor,
            vSheenRoughness,
            roughness,
        #ifdef SHEEN_TEXTURE
            vSheenUV,
            vSheenInfos,
            uvOffset,
            sheenSampler,
        #endif
            reflectance,
        #ifdef SHEEN_LINKWITHALBEDO
            baseColor,
            surfaceAlbedo,
        #endif
        #ifdef ENVIRONMENTBRDF
            NdotV,
        #endif
        #if defined(REFLECTION) && defined(ENVIRONMENTBRDF)
            AARoughnessFactors,
            vReflectionMicrosurfaceInfos,
            vReflectionInfos,
            vReflectionColor,
            vLightingIntensity,
            reflectionSampler,
            reflectionOut.reflectionCoords,
            NdotVUnclamped,
            #ifndef LODBASEDMICROSFURACE
                reflectionSamplerLow,
                reflectionSamplerHigh,
            #endif
            environmentBrdf,
            #if !defined(REFLECTIONMAP_SKYBOX) && defined(RADIANCEOCCLUSION)
                seo,
            #endif
            #if !defined(REFLECTIONMAP_SKYBOX) && defined(HORIZONOCCLUSION) && defined(BUMP) && defined(REFLECTIONMAP_3D)
                eho,
            #endif
        #endif
            sheenOut
        );

        #ifdef SHEEN_LINKWITHALBEDO
            surfaceAlbedo = sheenOut.surfaceAlbedo;
        #endif
    #endif

    // _____________________________ Clear Coat Information ____________________________
    #ifdef CLEARCOAT
        // Clear COAT parameters.
        float clearCoatIntensity = vClearCoatParams.x;
        float clearCoatRoughness = vClearCoatParams.y;

        #ifdef CLEARCOAT_TEXTURE
            vec2 clearCoatMapData = texture2D(clearCoatSampler, vClearCoatUV + uvOffset).rg * vClearCoatInfos.y;
            clearCoatIntensity *= clearCoatMapData.x;
            clearCoatRoughness *= clearCoatMapData.y;
        #endif

        #ifdef CLEARCOAT_TINT
            vec3 clearCoatColor = vClearCoatTintParams.rgb;
            float clearCoatThickness = vClearCoatTintParams.a;

            #ifdef CLEARCOAT_TINT_TEXTURE
                vec4 clearCoatTintMapData = texture2D(clearCoatTintSampler, vClearCoatTintUV + uvOffset);
                clearCoatColor *= toLinearSpace(clearCoatTintMapData.rgb);
                clearCoatThickness *= clearCoatTintMapData.a;
            #endif

            clearCoatColor = computeColorAtDistanceInMedia(clearCoatColor, clearCoatColorAtDistance);
        #endif

        // remapping and linearization of clear coat roughness
        // Let s see how it ends up in gltf
        // clearCoatRoughness = mix(0.089, 0.6, clearCoatRoughness);

        // Remap F0 to account for the change of interface within the material.
        vec3 specularEnvironmentR0Updated = getR0RemappedForClearCoat(specularEnvironmentR0);
        specularEnvironmentR0 = mix(specularEnvironmentR0, specularEnvironmentR0Updated, clearCoatIntensity);

        // Needs to use the geometric normal before bump for this.
        vec3 clearCoatNormalW = geometricNormalW;

        #ifdef CLEARCOAT_BUMP

            #ifdef NORMALXYSCALE
                float clearCoatNormalScale = 1.0;
            #else
                float clearCoatNormalScale = vClearCoatBumpInfos.y;
            #endif

            #if defined(TANGENT) && defined(NORMAL)
                mat3 TBNClearCoat = vTBN;
            #else
                mat3 TBNClearCoat = cotangent_frame(clearCoatNormalW * clearCoatNormalScale, vPositionW, vClearCoatBumpUV, vClearCoatTangentSpaceParams);
            #endif

            #ifdef OBJECTSPACE_NORMALMAP
                clearCoatNormalW = normalize(texture2D(clearCoatBumpSampler, vClearCoatBumpUV + uvOffset).xyz  * 2.0 - 1.0);
                clearCoatNormalW = normalize(mat3(normalMatrix) * clearCoatNormalW);
            #else
                clearCoatNormalW = perturbNormal(TBNClearCoat, texture2D(clearCoatBumpSampler, vClearCoatBumpUV + uvOffset).xyz, vClearCoatBumpInfos.y);
            #endif
        #endif

        #if defined(FORCENORMALFORWARD) && defined(NORMAL)
            clearCoatNormalW *= sign(dot(clearCoatNormalW, faceNormal));
        #endif

        #if defined(TWOSIDEDLIGHTING) && defined(NORMAL)
            clearCoatNormalW = gl_FrontFacing ? clearCoatNormalW : -clearCoatNormalW;
        #endif

        // Clear Coat AA
        vec2 clearCoatAARoughnessFactors = getAARoughnessFactors(clearCoatNormalW.xyz);

        // Compute N dot V.
        float clearCoatNdotVUnclamped = dot(clearCoatNormalW, viewDirectionW);
        // The order 1886 page 3.
        float clearCoatNdotV = absEps(clearCoatNdotVUnclamped);

        #ifdef CLEARCOAT_TINT
            // Used later on in the light fragment and ibl.
            vec3 clearCoatVRefract = -refract(vPositionW, clearCoatNormalW, vClearCoatRefractionParams.y);
            // The order 1886 page 3.
            float clearCoatNdotVRefract = absEps(dot(clearCoatNormalW, clearCoatVRefract));
            vec3 absorption = vec3(0.);
        #endif

        // Clear Coat Reflection
        #if defined(REFLECTION)
            float clearCoatAlphaG = convertRoughnessToAverageSlope(clearCoatRoughness);

            #ifdef SPECULARAA
                // Adapt linear roughness (alphaG) to geometric curvature of the current pixel.
                clearCoatAlphaG += clearCoatAARoughnessFactors.y;
            #endif

            vec4 environmentClearCoatRadiance = vec4(0., 0., 0., 0.);

            vec3 clearCoatReflectionVector = computeReflectionCoords(vec4(vPositionW, 1.0), clearCoatNormalW);
            #ifdef REFLECTIONMAP_OPPOSITEZ
                clearCoatReflectionVector.z *= -1.0;
            #endif

            // _____________________________ 2D vs 3D Maps ________________________________
            #ifdef REFLECTIONMAP_3D
                vec3 clearCoatReflectionCoords = clearCoatReflectionVector;
            #else
                vec2 clearCoatReflectionCoords = clearCoatReflectionVector.xy;
                #ifdef REFLECTIONMAP_PROJECTION
                    clearCoatReflectionCoords /= clearCoatReflectionVector.z;
                #endif
                clearCoatReflectionCoords.y = 1.0 - clearCoatReflectionCoords.y;
            #endif

            #if defined(LODINREFLECTIONALPHA) && !defined(REFLECTIONMAP_SKYBOX)
                float clearCoatReflectionLOD = getLodFromAlphaG(vReflectionMicrosurfaceInfos.x, clearCoatAlphaG, clearCoatNdotVUnclamped);
            #elif defined(LINEARSPECULARREFLECTION)
                float clearCoatReflectionLOD = getLinearLodFromRoughness(vReflectionMicrosurfaceInfos.x, clearCoatRoughness);
            #else
                float clearCoatReflectionLOD = getLodFromAlphaG(vReflectionMicrosurfaceInfos.x, clearCoatAlphaG);
            #endif

            #ifdef LODBASEDMICROSFURACE
                // Apply environment convolution scale/offset filter tuning parameters to the mipmap LOD selection
                clearCoatReflectionLOD = clearCoatReflectionLOD * vReflectionMicrosurfaceInfos.y + vReflectionMicrosurfaceInfos.z;
                float requestedClearCoatReflectionLOD = clearCoatReflectionLOD;

                environmentClearCoatRadiance = sampleReflectionLod(reflectionSampler, clearCoatReflectionCoords, requestedClearCoatReflectionLOD);
            #else
                float lodClearCoatReflectionNormalized = saturate(clearCoatReflectionLOD / log2(vReflectionMicrosurfaceInfos.x));
                float lodClearCoatReflectionNormalizedDoubled = lodClearCoatReflectionNormalized * 2.0;

                vec4 environmentClearCoatMid = sampleReflection(reflectionSampler, reflectionOut.reflectionCoords);
                if(lodClearCoatReflectionNormalizedDoubled < 1.0){
                    environmentClearCoatRadiance = mix(
                        sampleReflection(reflectionSamplerHigh, clearCoatReflectionCoords),
                        environmentClearCoatMid,
                        lodClearCoatReflectionNormalizedDoubled
                    );
                }else{
                    environmentClearCoatRadiance = mix(
                        environmentClearCoatMid,
                        sampleReflection(reflectionSamplerLow, clearCoatReflectionCoords),
                        lodClearCoatReflectionNormalizedDoubled - 1.0
                    );
                }
            #endif

            #ifdef RGBDREFLECTION
                environmentClearCoatRadiance.rgb = fromRGBD(environmentClearCoatRadiance);
            #endif

            #ifdef GAMMAREFLECTION
                environmentClearCoatRadiance.rgb = toLinearSpace(environmentClearCoatRadiance.rgb);
            #endif

            // _____________________________ Levels _____________________________________
            environmentClearCoatRadiance.rgb *= vReflectionInfos.x;
            environmentClearCoatRadiance.rgb *= vReflectionColor.rgb;
        #endif
    #endif

    // _____________________________ IBL BRDF + Energy Cons ________________________________
    #if defined(ENVIRONMENTBRDF)
        #ifdef MS_BRDF_ENERGY_CONSERVATION
            vec3 energyConservationFactor = getEnergyConservationFactor(specularEnvironmentR0, environmentBrdf);
        #endif
    #endif

    // _________________________ Specular Environment Oclusion __________________________
    #if defined(ENVIRONMENTBRDF) && !defined(REFLECTIONMAP_SKYBOX)
        vec3 specularEnvironmentReflectance = getReflectanceFromBRDFLookup(specularEnvironmentR0, environmentBrdf);

        #ifdef RADIANCEOCCLUSION
            specularEnvironmentReflectance *= seo;
        #endif

        #ifdef HORIZONOCCLUSION
            #ifdef BUMP
                #ifdef REFLECTIONMAP_3D
                    specularEnvironmentReflectance *= eho;
                #endif
            #endif
        #endif
    #else
        // Jones implementation of a well balanced fast analytical solution.
        vec3 specularEnvironmentReflectance = getReflectanceFromAnalyticalBRDFLookup_Jones(NdotV, specularEnvironmentR0, specularEnvironmentR90, sqrt(microSurface));
    #endif

    // _________________________ Clear Coat Environment Oclusion __________________________
    #ifdef CLEARCOAT
        #if defined(ENVIRONMENTBRDF) && !defined(REFLECTIONMAP_SKYBOX)
            // BRDF Lookup
            vec3 environmentClearCoatBrdf = getBRDFLookup(clearCoatNdotV, clearCoatRoughness);
            vec3 clearCoatEnvironmentReflectance = getReflectanceFromBRDFLookup(vec3(vClearCoatRefractionParams.x), environmentClearCoatBrdf);

            #ifdef RADIANCEOCCLUSION
                float clearCoatSeo = environmentRadianceOcclusion(ambientMonochrome, clearCoatNdotVUnclamped);
                clearCoatEnvironmentReflectance *= clearCoatSeo;
            #endif

            #ifdef HORIZONOCCLUSION
                #ifdef BUMP
                    #ifdef REFLECTIONMAP_3D
                        float clearCoatEho = environmentHorizonOcclusion(-viewDirectionW, clearCoatNormalW, geometricNormalW);
                        clearCoatEnvironmentReflectance *= clearCoatEho;
                    #endif
                #endif
            #endif
        #else
            // Jones implementation of a well balanced fast analytical solution.
            vec3 clearCoatEnvironmentReflectance = getReflectanceFromAnalyticalBRDFLookup_Jones(clearCoatNdotV, vec3(1.), vec3(1.), sqrt(1. - clearCoatRoughness));
        #endif

        clearCoatEnvironmentReflectance *= clearCoatIntensity;

        #if defined(CLEARCOAT_TINT)
            // NdotL = NdotV in IBL
            absorption = computeClearCoatAbsorption(clearCoatNdotVRefract, clearCoatNdotVRefract, clearCoatColor, clearCoatThickness, clearCoatIntensity);

            #ifdef REFLECTION
                reflectionOut.environmentIrradiance *= absorption;
            #endif

            #if defined(SHEEN) && defined(REFLECTION) && defined(ENVIRONMENTBRDF)
                sheenOut.finalSheenRadiance *= absorption;
            #endif

            specularEnvironmentReflectance *= absorption;
        #endif

        // clear coat energy conservation
        float fresnelIBLClearCoat = fresnelSchlickGGX(clearCoatNdotV, vClearCoatRefractionParams.x, CLEARCOATREFLECTANCE90);
        fresnelIBLClearCoat *= clearCoatIntensity;

        float conservationFactor = (1. - fresnelIBLClearCoat);

        #ifdef REFLECTION
            reflectionOut.environmentIrradiance *= conservationFactor;
        #endif

        #if defined(SHEEN) && defined(REFLECTION) && defined(ENVIRONMENTBRDF)
            sheenOut.finalSheenRadiance *= conservationFactor;
        #endif

        specularEnvironmentReflectance *= conservationFactor;

        #if defined(ENVIRONMENTBRDF) && defined(MS_BRDF_ENERGY_CONSERVATION)
            vec3 energyConservationFactorClearCoat = getEnergyConservationFactor(specularEnvironmentR0, environmentClearCoatBrdf);
        #endif
    #endif

    // ___________________________________ SubSurface ______________________________________
    subSurfaceOutParams subSurfaceOut;

    #ifdef SUBSURFACE
        subSurfaceBlock(
            vThicknessParam,
            vTintColor,
            normalW,
            specularEnvironmentReflectance,
        #ifdef SS_THICKNESSANDMASK_TEXTURE
            vThicknessUV,
            uvOffset,
            thicknessSampler,
        #endif
        #ifdef REFLECTION
            reflectionOut.environmentIrradiance,
            #ifdef SS_TRANSLUCENCY
                reflectionMatrix,
                #ifdef USESPHERICALFROMREFLECTIONMAP
                    #if !defined(NORMAL) || !defined(USESPHERICALINVERTEX)
                        reflectionOut.irradianceVector,
                    #endif
                #endif
                #ifdef USEIRRADIANCEMAP
                    irradianceSampler,
                #endif
            #endif
        #endif
        #ifdef SS_REFRACTION
            vPositionW,
            viewDirectionW,
            view,
            surfaceAlbedo,
            vRefractionInfos,
            refractionMatrix,
            vRefractionMicrosurfaceInfos,
            #ifdef SS_LINKREFRACTIONTOTRANSPARENCY
                alpha,
            #endif
            #ifdef SS_LODINREFRACTIONALPHA
                NdotVUnclamped,
            #endif
            #ifdef SS_LINEARSPECULARREFRACTION
                roughness,
            #else
                alphaG,
            #endif
            refractionSampler,
            #ifndef LODBASEDMICROSFURACE
                refractionSamplerLow,
                refractionSamplerHigh,
            #endif
            #ifdef ANISOTROPIC
                anisotropicOut,
            #endif
        #endif
        #ifdef SS_TRANSLUCENCY
            vDiffusionDistance,
        #endif
            subSurfaceOut
        );

        #ifdef SS_REFRACTION
            surfaceAlbedo = subSurfaceOut.surfaceAlbedo;
            #ifdef SS_LINKREFRACTIONTOTRANSPARENCY
                alpha = subSurfaceOut.alpha;
            #endif
        #endif
    #else
        subSurfaceOut.specularEnvironmentReflectance = specularEnvironmentReflectance;
        #ifdef REFLECTION
            subSurfaceOut.finalIrradiance = reflectionOut.environmentIrradiance;
        #endif
    #endif

    // ____________________________________________________________________________________
    // _____________________________ Direct Lighting Info __________________________________
    vec3 diffuseBase = vec3(0., 0., 0.);
    #ifdef SPECULARTERM
        vec3 specularBase = vec3(0., 0., 0.);
    #endif
    #ifdef CLEARCOAT
        vec3 clearCoatBase = vec3(0., 0., 0.);
    #endif
    #ifdef SHEEN
        vec3 sheenBase = vec3(0., 0., 0.);
    #endif

    #ifdef LIGHTMAP
        vec4 lightmapColor = texture2D(lightmapSampler, vLightmapUV + uvOffset);

        #ifdef RGBDLIGHTMAP
            lightmapColor.rgb = fromRGBD(lightmapColor);
        #endif

        #ifdef GAMMALIGHTMAP
            lightmapColor.rgb = toLinearSpace(lightmapColor.rgb);
        #endif
        lightmapColor.rgb *= vLightmapInfos.y;
    #endif

    // Direct Lighting Variables
    preLightingInfo preInfo;
    lightingInfo info;
    float shadow = 1.; // 1 - shadowLevel

    #include<lightFragment>[0..maxSimultaneousLights]

    // ______________________________________________________________________________
    // _____________________________ Energy Conservation  ___________________________
    // Apply Energy Conservation.
    #ifndef METALLICWORKFLOW
        #ifdef SPECULAR_GLOSSINESS_ENERGY_CONSERVATION
            surfaceAlbedo.rgb = (1. - reflectance) * surfaceAlbedo.rgb;
        #endif
    #endif

    #if defined(SHEEN) && defined(SHEEN_ALBEDOSCALING) && defined(ENVIRONMENTBRDF)
        surfaceAlbedo.rgb = sheenOut.sheenAlbedoScaling * surfaceAlbedo.rgb;
    #endif

    // _____________________________ Irradiance ______________________________________
    #ifdef REFLECTION
        /*vec3 finalIrradiance = reflectionOut.environmentIrradiance;
        #if defined(SS_TRANSLUCENCY)
            finalIrradiance += refractionIrradiance;
        #endif*/
        subSurfaceOut.finalIrradiance *= surfaceAlbedo.rgb;
    #endif

    // _____________________________ Specular ________________________________________
    #ifdef SPECULARTERM
        vec3 finalSpecular = specularBase;
        finalSpecular = max(finalSpecular, 0.0);

        vec3 finalSpecularScaled = finalSpecular * vLightingIntensity.x * vLightingIntensity.w;
        #if defined(ENVIRONMENTBRDF) && defined(MS_BRDF_ENERGY_CONSERVATION)
            finalSpecularScaled *= energyConservationFactor;
        #endif

        #if defined(SHEEN) && defined(ENVIRONMENTBRDF) && defined(SHEEN_ALBEDOSCALING)
            finalSpecularScaled *= sheenOut.sheenAlbedoScaling;
        #endif
    #endif

    // _____________________________ Radiance ________________________________________
    #ifdef REFLECTION
        vec3 finalRadiance = reflectionOut.environmentRadiance.rgb;
        finalRadiance *= subSurfaceOut.specularEnvironmentReflectance;

        vec3 finalRadianceScaled = finalRadiance * vLightingIntensity.z;
        #if defined(ENVIRONMENTBRDF) && defined(MS_BRDF_ENERGY_CONSERVATION)
            finalRadianceScaled *= energyConservationFactor;
        #endif

        #if defined(SHEEN) && defined(ENVIRONMENTBRDF) && defined(SHEEN_ALBEDOSCALING)
            finalRadianceScaled *= sheenOut.sheenAlbedoScaling;
        #endif
    #endif

    // ________________________________ Sheen ________________________________________
    #ifdef SHEEN
        vec3 finalSheen = sheenBase * sheenOut.sheenColor;
        finalSheen = max(finalSheen, 0.0);

        vec3 finalSheenScaled = finalSheen * vLightingIntensity.x * vLightingIntensity.w;
        // #if defined(ENVIRONMENTBRDF) && defined(MS_BRDF_ENERGY_CONSERVATION)
            // The sheen does not use the same BRDF so not energy conservation is possible
            // Should be less a problem as it is usually not metallic
            // finalSheenScaled *= energyConservationFactor;
        // #endif
    #endif

    // _____________________________ Clear Coat _______________________________________
    #ifdef CLEARCOAT
        vec3 finalClearCoat = clearCoatBase;
        finalClearCoat = max(finalClearCoat, 0.0);

        // Full value needed for alpha.
        vec3 finalClearCoatScaled = finalClearCoat * vLightingIntensity.x * vLightingIntensity.w;
        #if defined(ENVIRONMENTBRDF) && defined(MS_BRDF_ENERGY_CONSERVATION)
            finalClearCoatScaled *= energyConservationFactorClearCoat;
        #endif

    // ____________________________ Clear Coat Radiance _______________________________
        #ifdef REFLECTION
            vec3 finalClearCoatRadiance = environmentClearCoatRadiance.rgb;
            finalClearCoatRadiance *= clearCoatEnvironmentReflectance;

            // Full value needed for alpha. 
            vec3 finalClearCoatRadianceScaled = finalClearCoatRadiance * vLightingIntensity.z;
        #endif

        #ifdef SS_REFRACTION
            subSurfaceOut.finalRefraction *= conservationFactor;
            #ifdef CLEARCOAT_TINT
                subSurfaceOut.finalRefraction *= absorption;
            #endif
        #endif
    #endif

    // _____________________________ Highlights on Alpha _____________________________
    #ifdef ALPHABLEND
        float luminanceOverAlpha = 0.0;
        #if	defined(REFLECTION) && defined(RADIANCEOVERALPHA)
            luminanceOverAlpha += getLuminance(finalRadianceScaled);
            #if defined(CLEARCOAT)
                luminanceOverAlpha += getLuminance(finalClearCoatRadianceScaled);
            #endif
        #endif

        #if defined(SPECULARTERM) && defined(SPECULAROVERALPHA)
            luminanceOverAlpha += getLuminance(finalSpecularScaled);
        #endif

        #if defined(CLEARCOAT) && defined(CLEARCOATOVERALPHA)
            luminanceOverAlpha += getLuminance(finalClearCoatScaled);
        #endif

        #if defined(RADIANCEOVERALPHA) || defined(SPECULAROVERALPHA)
            alpha = saturate(alpha + luminanceOverAlpha * luminanceOverAlpha);
        #endif
    #endif
#endif

// _______________ Not done before as it is unlit only __________________________
// _____________________________ Diffuse ________________________________________
    vec3 finalDiffuse = diffuseBase;
    finalDiffuse *= surfaceAlbedo.rgb;
    finalDiffuse = max(finalDiffuse, 0.0);

// _____________________________ Ambient ________________________________________
    vec3 finalAmbient = vAmbientColor;
    finalAmbient *= surfaceAlbedo.rgb;

// _____________________________ Emissive ________________________________________
    vec3 finalEmissive = vEmissiveColor;
#ifdef EMISSIVE
    vec3 emissiveColorTex = texture2D(emissiveSampler, vEmissiveUV + uvOffset).rgb;
    finalEmissive *= toLinearSpace(emissiveColorTex.rgb);
    finalEmissive *=  vEmissiveInfos.y;
#endif

// ______________________________ Ambient ________________________________________
#ifdef AMBIENT
    vec3 ambientOcclusionForDirectDiffuse = mix(vec3(1.), aoOut.ambientOcclusionColor, vAmbientInfos.w);
#else
    vec3 ambientOcclusionForDirectDiffuse = aoOut.ambientOcclusionColor;
#endif

// _______________________________________________________________________________
// _____________________________ Composition _____________________________________
    // Reflection already includes the environment intensity.
    vec4 finalColor = vec4(
        finalAmbient			* aoOut.ambientOcclusionColor +
        finalDiffuse			* ambientOcclusionForDirectDiffuse * vLightingIntensity.x +
#ifndef UNLIT
    #ifdef REFLECTION
        subSurfaceOut.finalIrradiance * aoOut.ambientOcclusionColor * vLightingIntensity.z +
    #endif
    #ifdef SPECULARTERM
    // Computed in the previous step to help with alpha luminance.
    //	finalSpecular			* vLightingIntensity.x * vLightingIntensity.w +
        finalSpecularScaled +
    #endif
    #ifdef SHEEN
    // Computed in the previous step to help with alpha luminance.
    //	finalSheen  			* vLightingIntensity.x * vLightingIntensity.w +
        finalSheenScaled +
    #endif
    #ifdef CLEARCOAT
    // Computed in the previous step to help with alpha luminance.
    //	finalClearCoat			* vLightingIntensity.x * vLightingIntensity.w +
        finalClearCoatScaled +
    #endif
    #ifdef REFLECTION
    // Comupted in the previous step to help with alpha luminance.
    //	finalRadiance			* vLightingIntensity.z +
        finalRadianceScaled +
        #if defined(SHEEN) && defined(ENVIRONMENTBRDF)
        //  Comupted in the previous step to help with alpha luminance.
        //  finalSheenRadiance * vLightingIntensity.z 
            sheenOut.finalSheenRadiance +
        #endif
        #ifdef CLEARCOAT
        //  Comupted in the previous step to help with alpha luminance.
        //  finalClearCoatRadiance * vLightingIntensity.z 
            finalClearCoatRadianceScaled +
        #endif
    #endif
    #ifdef SS_REFRACTION
        subSurfaceOut.finalRefraction * vLightingIntensity.z +
    #endif
#endif
        finalEmissive			* vLightingIntensity.y,
        alpha);

// _____________________________ LightMappping _____________________________________
#ifdef LIGHTMAP
    #ifndef LIGHTMAPEXCLUDED
        #ifdef USELIGHTMAPASSHADOWMAP
            finalColor.rgb *= lightmapColor.rgb;
        #else
            finalColor.rgb += lightmapColor.rgb;
        #endif
    #endif
#endif

    #define CUSTOM_FRAGMENT_BEFORE_FOG

// _____________________________ Finally ___________________________________________
    finalColor = max(finalColor, 0.0);

    #include<logDepthFragment>
    #include<fogFragment>(color, finalColor)

#ifdef IMAGEPROCESSINGPOSTPROCESS
    // Sanitize output incase invalid normals or tangents have caused div by 0 or undefined behavior
    // this also limits the brightness which helpfully reduces over-sparkling in bloom (native handles this in the bloom blur shader)
    finalColor.rgb = clamp(finalColor.rgb, 0., 30.0);
#else
    // Alway run to ensure we are going back to gamma space.
    finalColor = applyImageProcessing(finalColor);
#endif

    finalColor.a *= visibility;

#ifdef PREMULTIPLYALPHA
    // Convert to associative (premultiplied) format if needed.
    finalColor.rgb *= finalColor.a;
#endif

    #define CUSTOM_FRAGMENT_BEFORE_FRAGCOLOR

    gl_FragColor = finalColor;

    #include<pbrDebug>
}
